# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'activate/resources/ui/main.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_main_window(object):
    def setupUi(self, main_window):
        main_window.setObjectName("main_window")
        main_window.resize(770, 507)
        self.main_widget = QtWidgets.QWidget(main_window)
        self.main_widget.setObjectName("main_widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.main_widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.main_tabs = QtWidgets.QTabWidget(self.main_widget)
        self.main_tabs.setObjectName("main_tabs")
        self.summary_tab = QtWidgets.QWidget()
        self.summary_tab.setObjectName("summary_tab")
        self.summary_tab_layout = QtWidgets.QHBoxLayout(self.summary_tab)
        self.summary_tab_layout.setObjectName("summary_tab_layout")
        self.summary = Summary(self.summary_tab)
        self.summary.setObjectName("summary")
        self.summary_tab_layout.addWidget(self.summary)
        self.main_tabs.addTab(self.summary_tab, "")
        self.activities_tab = QtWidgets.QWidget()
        self.activities_tab.setObjectName("activities_tab")
        self.activities_tab_layout = QtWidgets.QHBoxLayout(self.activities_tab)
        self.activities_tab_layout.setObjectName("activities_tab_layout")
        self.activity_list_area = QtWidgets.QVBoxLayout()
        self.activity_list_area.setObjectName("activity_list_area")
        self.activities_label = QtWidgets.QLabel(self.activities_tab)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.activities_label.setFont(font)
        self.activities_label.setObjectName("activities_label")
        self.activity_list_area.addWidget(self.activities_label)
        self.activity_list_table = ActivityListTable(self.activities_tab)
        self.activity_list_table.setMinimumSize(QtCore.QSize(500, 0))
        self.activity_list_table.setMaximumSize(QtCore.QSize(600, 16777215))
        self.activity_list_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.activity_list_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.activity_list_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.activity_list_table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.activity_list_table.setCornerButtonEnabled(False)
        self.activity_list_table.setObjectName("activity_list_table")
        self.activity_list_table.setColumnCount(0)
        self.activity_list_table.setRowCount(0)
        self.activity_list_area.addWidget(self.activity_list_table)
        self.activities_tab_layout.addLayout(self.activity_list_area)
        self.activity_summary = ActivitySummary(self.activities_tab)
        self.activity_summary.setObjectName("activity_summary")
        self.activities_tab_layout.addWidget(self.activity_summary)
        self.main_tabs.addTab(self.activities_tab, "")
        self.social_tab = QtWidgets.QWidget()
        self.social_tab.setObjectName("social_tab")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.social_tab)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.social_tree = SocialTree(self.social_tab)
        self.social_tree.setObjectName("social_tree")
        self.social_tree.headerItem().setText(0, "1")
        self.verticalLayout.addWidget(self.social_tree)
        self.social_activity_list = SocialActivityList(self.social_tab)
        self.social_activity_list.setMinimumSize(QtCore.QSize(720, 0))
        self.social_activity_list.setMaximumSize(QtCore.QSize(600, 16777215))
        self.social_activity_list.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.social_activity_list.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.social_activity_list.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.social_activity_list.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.social_activity_list.setCornerButtonEnabled(False)
        self.social_activity_list.setObjectName("social_activity_list")
        self.social_activity_list.setColumnCount(0)
        self.social_activity_list.setRowCount(0)
        self.verticalLayout.addWidget(self.social_activity_list)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.social_activity_summary = ActivitySummary(self.social_tab)
        self.social_activity_summary.setObjectName("social_activity_summary")
        self.horizontalLayout_2.addWidget(self.social_activity_summary)
        self.main_tabs.addTab(self.social_tab, "")
        self.horizontalLayout.addWidget(self.main_tabs)
        main_window.setCentralWidget(self.main_widget)
        self.menubar = QtWidgets.QMenuBar(main_window)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 770, 30))
        self.menubar.setObjectName("menubar")
        self.file_menu = QtWidgets.QMenu(self.menubar)
        self.file_menu.setObjectName("file_menu")
        self.export_menu = QtWidgets.QMenu(self.file_menu)
        self.export_menu.setObjectName("export_menu")
        self.settings_menu = QtWidgets.QMenu(self.menubar)
        self.settings_menu.setObjectName("settings_menu")
        self.activity_menu = QtWidgets.QMenu(self.menubar)
        self.activity_menu.setObjectName("activity_menu")
        main_window.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(main_window)
        self.statusbar.setObjectName("statusbar")
        main_window.setStatusBar(self.statusbar)
        self.action_import = QtWidgets.QAction(main_window)
        self.action_import.setObjectName("action_import")
        self.action_quit = QtWidgets.QAction(main_window)
        self.action_quit.setObjectName("action_quit")
        self.action_units = QtWidgets.QAction(main_window)
        self.action_units.setObjectName("action_units")
        self.action_edit = QtWidgets.QAction(main_window)
        self.action_edit.setObjectName("action_edit")
        self.export_original = QtWidgets.QAction(main_window)
        self.export_original.setObjectName("export_original")
        self.action_add_manual = QtWidgets.QAction(main_window)
        self.action_add_manual.setObjectName("action_add_manual")
        self.action_add_photos = QtWidgets.QAction(main_window)
        self.action_add_photos.setObjectName("action_add_photos")
        self.actionAdd = QtWidgets.QAction(main_window)
        self.actionAdd.setObjectName("actionAdd")
        self.action_servers = QtWidgets.QAction(main_window)
        self.action_servers.setObjectName("action_servers")
        self.action_analyse = QtWidgets.QAction(main_window)
        self.action_analyse.setObjectName("action_analyse")
        self.action_match = QtWidgets.QAction(main_window)
        self.action_match.setObjectName("action_match")
        self.action_as_route = QtWidgets.QAction(main_window)
        self.action_as_route.setObjectName("action_as_route")
        self.export_menu.addAction(self.export_original)
        self.export_menu.addAction(self.action_as_route)
        self.file_menu.addAction(self.action_import)
        self.file_menu.addAction(self.action_add_manual)
        self.file_menu.addAction(self.export_menu.menuAction())
        self.file_menu.addSeparator()
        self.file_menu.addAction(self.action_quit)
        self.settings_menu.addAction(self.action_units)
        self.settings_menu.addAction(self.action_servers)
        self.activity_menu.addAction(self.action_edit)
        self.activity_menu.addAction(self.action_add_photos)
        self.activity_menu.addAction(self.action_analyse)
        self.activity_menu.addAction(self.action_match)
        self.menubar.addAction(self.file_menu.menuAction())
        self.menubar.addAction(self.activity_menu.menuAction())
        self.menubar.addAction(self.settings_menu.menuAction())

        self.retranslateUi(main_window)
        self.main_tabs.setCurrentIndex(0)
        self.action_quit.triggered.connect(main_window.close)
        self.action_import.triggered.connect(main_window.import_activities)
        self.action_units.triggered.connect(main_window.edit_unit_settings)
        self.action_edit.triggered.connect(main_window.edit_activity_data)
        self.export_original.triggered.connect(main_window.export_activity)
        self.action_add_manual.triggered.connect(main_window.add_manual_activity)
        self.action_add_photos.triggered.connect(main_window.add_photos)
        self.action_servers.triggered.connect(main_window.edit_server_settings)
        self.action_analyse.triggered.connect(main_window.analyse_activity)
        self.activity_list_table.currentCellChanged['int','int','int','int'].connect(main_window.update_activity)
        self.main_tabs.currentChanged['int'].connect(main_window.main_tab_switch)
        self.social_tree.itemChanged['QTreeWidgetItem*','int'].connect(main_window.filter_social_activities)
        self.social_activity_list.currentCellChanged['int','int','int','int'].connect(main_window.update_social_activity)
        self.action_match.triggered.connect(main_window.match_activity)
        self.action_as_route.triggered.connect(main_window.export_as_route)
        QtCore.QMetaObject.connectSlotsByName(main_window)

    def retranslateUi(self, main_window):
        _translate = QtCore.QCoreApplication.translate
        self.main_tabs.setTabText(self.main_tabs.indexOf(self.summary_tab), _translate("main_window", "Summary"))
        self.activities_label.setText(_translate("main_window", "Activities"))
        self.activity_list_table.setSortingEnabled(True)
        self.main_tabs.setTabText(self.main_tabs.indexOf(self.activities_tab), _translate("main_window", "Activities"))
        self.social_activity_list.setSortingEnabled(True)
        self.main_tabs.setTabText(self.main_tabs.indexOf(self.social_tab), _translate("main_window", "Social"))
        self.file_menu.setTitle(_translate("main_window", "File"))
        self.export_menu.setTitle(_translate("main_window", "Export"))
        self.settings_menu.setTitle(_translate("main_window", "Settings"))
        self.activity_menu.setTitle(_translate("main_window", "Activity"))
        self.action_import.setText(_translate("main_window", "Import"))
        self.action_import.setShortcut(_translate("main_window", "Ctrl+I"))
        self.action_quit.setText(_translate("main_window", "Quit"))
        self.action_units.setText(_translate("main_window", "Units"))
        self.action_edit.setText(_translate("main_window", "Edit"))
        self.export_original.setText(_translate("main_window", "Original"))
        self.action_add_manual.setText(_translate("main_window", "Add Manual"))
        self.action_add_manual.setShortcut(_translate("main_window", "Ctrl+M"))
        self.action_add_photos.setText(_translate("main_window", "Add Photos"))
        self.actionAdd.setText(_translate("main_window", "Add"))
        self.action_servers.setText(_translate("main_window", "Servers"))
        self.action_analyse.setText(_translate("main_window", "Analyse"))
        self.action_match.setText(_translate("main_window", "Find Matching"))
        self.action_as_route.setText(_translate("main_window", "As Route"))
from activate.app.activity_summary import ActivitySummary
from activate.app.social_tree import SocialTree
from activate.app.summary import Summary
from activate.app.tables import ActivityListTable, SocialActivityList



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'activate/resources/ui/activity_view.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_activity_view(object):
    def setupUi(self, activity_view):
        activity_view.setObjectName("activity_view")
        activity_view.resize(567, 254)
        self.activity_view_layout = QtWidgets.QHBoxLayout(activity_view)
        self.activity_view_layout.setObjectName("activity_view_layout")
        self.map_container = QtWidgets.QGridLayout()
        self.map_container.setObjectName("map_container")
        self.activity_view_layout.addLayout(self.map_container)
        self.activity_tabs = QtWidgets.QTabWidget(activity_view)
        self.activity_tabs.setObjectName("activity_tabs")
        self.overview_tab = QtWidgets.QWidget()
        self.overview_tab.setObjectName("overview_tab")
        self.overview_tab_layout = QtWidgets.QHBoxLayout(self.overview_tab)
        self.overview_tab_layout.setObjectName("overview_tab_layout")
        self.activity_text_area = QtWidgets.QVBoxLayout()
        self.activity_text_area.setObjectName("activity_text_area")
        self.activity_name_label = QtWidgets.QLabel(self.overview_tab)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.activity_name_label.sizePolicy().hasHeightForWidth()
        )
        self.activity_name_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.activity_name_label.setFont(font)
        self.activity_name_label.setObjectName("activity_name_label")
        self.activity_text_area.addWidget(self.activity_name_label)
        self.flags_label = QtWidgets.QLabel(self.overview_tab)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.flags_label.setFont(font)
        self.flags_label.setObjectName("flags_label")
        self.activity_text_area.addWidget(self.flags_label)
        self.date_time_label = QtWidgets.QLabel(self.overview_tab)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.date_time_label.setFont(font)
        self.date_time_label.setObjectName("date_time_label")
        self.activity_text_area.addWidget(self.date_time_label)
        self.activity_type_label = QtWidgets.QLabel(self.overview_tab)
        self.activity_type_label.setMaximumSize(QtCore.QSize(120, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.activity_type_label.setFont(font)
        self.activity_type_label.setObjectName("activity_type_label")
        self.activity_text_area.addWidget(self.activity_type_label)
        self.description_label = QtWidgets.QLabel(self.overview_tab)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.description_label.sizePolicy().hasHeightForWidth()
        )
        self.description_label.setSizePolicy(sizePolicy)
        self.description_label.setAlignment(
            QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop
        )
        self.description_label.setWordWrap(True)
        self.description_label.setObjectName("description_label")
        self.activity_text_area.addWidget(self.description_label)
        self.activity_text_area.setStretch(4, 1)
        self.overview_tab_layout.addLayout(self.activity_text_area)
        self.info_table = InfoTable(self.overview_tab)
        self.info_table.setMinimumSize(QtCore.QSize(310, 0))
        self.info_table.setMaximumSize(QtCore.QSize(310, 16777215))
        self.info_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.info_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.info_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.info_table.setShowGrid(False)
        self.info_table.setColumnCount(3)
        self.info_table.setObjectName("info_table")
        self.info_table.setRowCount(0)
        self.info_table.horizontalHeader().setVisible(False)
        self.info_table.horizontalHeader().setDefaultSectionSize(100)
        self.info_table.horizontalHeader().setStretchLastSection(True)
        self.info_table.verticalHeader().setVisible(False)
        self.overview_tab_layout.addWidget(self.info_table)
        self.activity_tabs.addTab(self.overview_tab, "")
        self.data_tab = QtWidgets.QWidget()
        self.data_tab.setMinimumSize(QtCore.QSize(100, 0))
        self.data_tab.setObjectName("data_tab")
        self.data_tab_layout = QtWidgets.QVBoxLayout(self.data_tab)
        self.data_tab_layout.setObjectName("data_tab_layout")
        self.graphs_layout = QtWidgets.QVBoxLayout()
        self.graphs_layout.setObjectName("graphs_layout")
        self.data_tab_layout.addLayout(self.graphs_layout)
        self.activity_tabs.addTab(self.data_tab, "")
        self.splits_tab = QtWidgets.QWidget()
        self.splits_tab.setObjectName("splits_tab")
        self.splits_tab_layout = QtWidgets.QVBoxLayout(self.splits_tab)
        self.splits_tab_layout.setObjectName("splits_tab_layout")
        self.split_table = SplitTable(self.splits_tab)
        self.split_table.setMinimumSize(QtCore.QSize(210, 0))
        self.split_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.split_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.split_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.split_table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.split_table.setObjectName("split_table")
        self.split_table.setColumnCount(0)
        self.split_table.setRowCount(0)
        self.split_table.horizontalHeader().setSortIndicatorShown(True)
        self.split_table.verticalHeader().setVisible(False)
        self.splits_tab_layout.addWidget(self.split_table)
        self.activity_tabs.addTab(self.splits_tab, "")
        self.zones_tab = QtWidgets.QWidget()
        self.zones_tab.setObjectName("zones_tab")
        self.zones_tab_layout = QtWidgets.QVBoxLayout(self.zones_tab)
        self.zones_tab_layout.setObjectName("zones_tab_layout")
        self.zones_graph = QtChart.QChartView(self.zones_tab)
        self.zones_graph.setObjectName("zones_graph")
        self.zones_tab_layout.addWidget(self.zones_graph)
        self.activity_tabs.addTab(self.zones_tab, "")
        self.curve_tab = QtWidgets.QWidget()
        self.curve_tab.setObjectName("curve_tab")
        self.curve_tab_layout = QtWidgets.QHBoxLayout(self.curve_tab)
        self.curve_tab_layout.setObjectName("curve_tab_layout")
        self.curve_graph = QtChart.QChartView(self.curve_tab)
        self.curve_graph.setObjectName("curve_graph")
        self.curve_tab_layout.addWidget(self.curve_graph)
        self.curve_table = CurveTable(self.curve_tab)
        self.curve_table.setMaximumSize(QtCore.QSize(320, 16777215))
        self.curve_table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.curve_table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.curve_table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.curve_table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.curve_table.setObjectName("curve_table")
        self.curve_table.setColumnCount(0)
        self.curve_table.setRowCount(0)
        self.curve_table.horizontalHeader().setStretchLastSection(True)
        self.curve_table.verticalHeader().setVisible(False)
        self.curve_tab_layout.addWidget(self.curve_table)
        self.activity_tabs.addTab(self.curve_tab, "")
        self.activity_view_layout.addWidget(self.activity_tabs)

        self.retranslateUi(activity_view)
        self.activity_tabs.setCurrentIndex(0)
        self.activity_tabs.currentChanged["int"].connect(activity_view.update_page)
        QtCore.QMetaObject.connectSlotsByName(activity_view)

    def retranslateUi(self, activity_view):
        _translate = QtCore.QCoreApplication.translate
        activity_view.setWindowTitle(_translate("activity_view", "Analyse Activity"))
        self.activity_name_label.setText(_translate("activity_view", "[Activity Name]"))
        self.flags_label.setText(_translate("activity_view", "[Flags]"))
        self.date_time_label.setText(_translate("activity_view", "[Date]"))
        self.activity_type_label.setText(_translate("activity_view", "[Type]"))
        self.description_label.setText(_translate("activity_view", "[Description]"))
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.overview_tab),
            _translate("activity_view", "Overview"),
        )
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.data_tab),
            _translate("activity_view", "Data"),
        )
        self.split_table.setSortingEnabled(True)
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.splits_tab),
            _translate("activity_view", "Splits"),
        )
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.zones_tab),
            _translate("activity_view", "Zones"),
        )
        self.activity_tabs.setTabText(
            self.activity_tabs.indexOf(self.curve_tab),
            _translate("activity_view", "Curve"),
        )


from PyQt5 import QtChart

from activate.app.tables import CurveTable, InfoTable, SplitTable



from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import Qt

from activate.app import paths


class ClickablePhoto(QtWidgets.QLabel):
    """
    A photo that can be clicked.

    It may also have a right-click menu for deleting itself.
    """

    clicked = QtCore.pyqtSignal(int)
    deleted = QtCore.pyqtSignal(int)

    def __init__(self, *args, deletable=False, **kwargs):
        super().__init__(*args, **kwargs)
        self.menu = QtWidgets.QMenu(self)
        self._index = None
        if deletable:
            self.action_delete = QtWidgets.QAction("Delete")
            self.action_delete.setIcon(QtGui.QIcon.fromTheme("edit-delete"))
            self.action_delete.triggered.connect(self.delete)
            self.menu.addAction(self.action_delete)

    @property
    def index(self):
        if self._index is None:
            return self.parent().layout().indexOf(self)
        return self._index

    @index.setter
    def index(self, value):
        self._index = value

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.clicked.emit(self.index)
            super().mousePressEvent(event)

    def contextMenuEvent(self, event):
        self.menu.exec(event.globalPos())

    def delete(self):
        self.deleted.emit(self.index)


class Gallery(QtWidgets.QWidget):
    """A multi-row container for photos."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLayout = QtWidgets.QVBoxLayout(self)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.width = event.size().width()
        self.height = event.size().height()

    def empty(self):
        """Remove all photos in the PhotoList."""
        for index in range(self.layout().count() - 1, -1, -1):
            self.layout().itemAt(index).widget().setParent(None)

    def replace_photos(self, filenames):
        """Replace the photos with new ones."""
        self.filenames = filenames
        self.empty()
        row_height = 120
        self.photos = [
            QtGui.QPixmap(f).scaledToHeight(row_height, Qt.SmoothTransformation)
            for f in filenames
        ]
        row = QtWidgets.QWidget(self)
        row.setLayout(QtWidgets.QHBoxLayout(row))
        width = self.space(row, 0)
        for index, photo in enumerate(self.photos):
            next_photo_width = photo.width()
            width += next_photo_width + row.layout().spacing()
            if width > self.width:
                self.layout().addWidget(row)
                row = QtWidgets.QWidget(self)
                row.setLayout(QtWidgets.QHBoxLayout(row))
                width = self.space(row, 1) + next_photo_width
            photo = self.create_label(photo)
            photo.index = index
            row.layout().addWidget(photo)
        self.layout().addWidget(row)

    def create_label(self, photo):
        label = ClickablePhoto(self, deletable=False)
        label.setPixmap(photo)
        label.clicked.connect(self.show_photos)
        return label

    def space(self, row, photo_count):
        """Get the required empty space for photo_count photos in row."""
        return (
            row.layout().spacing() * (photo_count - 1)
            + self.layout().contentsMargins().left()
            + self.layout().contentsMargins().right()
            + row.layout().contentsMargins().left()
            + row.layout().contentsMargins().right()
        )

    def show_photos(self, index):
        """Open the photo viewer on a photo."""
        viewer = PhotoViewer(self.filenames, index)
        viewer.exec()


class PhotoList(QtWidgets.QWidget):
    """A container for photos."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLayout = QtWidgets.QHBoxLayout(self)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.width = event.size().width()
        self.height = event.size().height()

    def empty(self):
        """Remove all photos in the PhotoList."""
        for index in range(self.layout().count() - 1, -1, -1):
            self.layout().itemAt(index).widget().setParent(None)

    def replace_photos(self, filenames):
        """Replace the photos with new ones."""
        self.empty()
        self.filenames = filenames
        if not filenames:
            return
        self.photos = [QtGui.QPixmap(str(f)) for f in filenames]
        total_aspect = sum(i.width() / i.height() for i in self.photos)
        total_aspect = max(total_aspect, 2)
        width = self.width - self.space(len(self.photos))
        height = width / total_aspect
        self.photos = [
            p.scaledToHeight(height, Qt.SmoothTransformation) for p in self.photos
        ]
        self.labels = []
        for photo in self.photos:
            self.add_photo(photo)

    def add_photo(self, photo):
        """Add a new photo to the list."""
        label = ClickablePhoto(self, deletable=True)
        label.setPixmap(photo)
        label.deleted.connect(self.delete)
        label.clicked.connect(self.show_photos)
        self.layout().addWidget(label)
        self.labels.append(label)

    def space(self, photo_count):
        """Get the required empty space for photo_count photos."""
        return (
            self.layout().spacing() * (photo_count - 1)
            + self.layout().contentsMargins().left()
            + self.layout().contentsMargins().right()
        )

    def show_photos(self, index):
        """Open the photo viewer on a photo."""
        viewer = PhotoViewer(self.filenames, index)
        viewer.exec()

    def show_activity_photos(self, activity):
        """Update to showing a new activity's photos."""
        self.activity = activity
        self.replace_photos(activity.photos)

    def delete(self, index):
        """Remove a photo from an activity."""
        self.filenames.pop(index)
        self.photos.pop(index)
        self.replace_photos(self.filenames)
        self.activity.photos = self.filenames
        self.activity.save(paths.ACTIVITIES)


class PhotoViewer(QtWidgets.QDialog):
    def __init__(self, photos, current_index, *args, **kwargs):
        self.filenames = photos.copy()
        self.current_index = current_index
        self.photos = photos
        super().__init__(*args, **kwargs)
        self.label = QtWidgets.QLabel(self)
        self.main_layout = QtWidgets.QHBoxLayout(self)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.addWidget(self.label, alignment=Qt.AlignCenter)
        self.show_photo()

    def show_photo(self):
        """Display a new photo.."""
        photo = self.photos[self.current_index]
        if not isinstance(photo, QtGui.QPixmap):
            photo = QtGui.QPixmap(photo)
            photo = photo.scaled(
                self.width(), self.height(), Qt.KeepAspectRatio, Qt.SmoothTransformation
            )
            self.photos[self.current_index] = photo
        self.label.setPixmap(photo)

    def set_new_index(self, index):
        """Handle left and right keypresses."""
        self.current_index = index
        self.current_index %= len(self.photos)
        self.show_photo()

    def keyPressEvent(self, event):
        if event.key() in {Qt.Key_Right, Qt.Key_Space}:
            self.set_new_index(self.current_index + 1)
            return
        if event.key() in {Qt.Key_Left, Qt.Key_Backspace}:
            self.set_new_index(self.current_index - 1)
            return
        super().keyPressEvent(event)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.photos = self.filenames.copy()
        self.show_photo()



"""Display an individual activity's data."""
import markdown
from PyQt5 import QtWidgets

from activate import times
from activate.app import photos
from activate.app.ui.activity_summary import Ui_activity_summary


class ActivitySummary(QtWidgets.QWidget, Ui_activity_summary):
    """A one-page activity summary."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)

    def setup(self, unit_system, map_widget):
        self.unit_system = unit_system
        self.map_widget = map_widget
        self.updated = set()
        self.photo_list = photos.PhotoList(self)
        self.activity_summary_layout.addWidget(self.photo_list, 1, 1)
        self.info_table.set_units(self.unit_system)

    def update(self):
        """Update labels, map and data box."""
        self.activity_name_label.setText(self.activity.name)
        self.flags_label.setText(" | ".join(self.activity.active_flags))
        self.description_label.setText(markdown.markdown(self.activity.description))
        self.date_time_label.setText(times.nice(self.activity.start_time))
        self.activity_type_label.setText(self.activity.sport)
        self.info_table.update_data(self.activity.stats)
        if self.activity.track.has_position_data:
            self.map_widget.setVisible(True)
            self.map_widget.show_route(self.activity.track.lat_lon_list)
            self.show_map()
        else:
            self.map_widget.setVisible(False)
        self.photo_list.show_activity_photos(self.activity)

    def show_activity(self, new_activity):
        """Display a new activity."""
        self.activity = new_activity
        self.update()

    def show_map(self):
        """
        Take back the map widget.

        This is necessary because the map widget must be shared between
        all layouts, and a widget cannot be in multiple places at once.
        Call this when the activity summary becomes visible.
        """
        self.map_container.addWidget(self.map_widget)



"""Functions for dealing with datetimes and timedeltas."""
from datetime import datetime, timedelta

ONE_WEEK = timedelta(days=7)
ONE_DAY = timedelta(days=1)
ONE_HOUR = timedelta(hours=1)
ONE_MINUTE = timedelta(minutes=1)

EPOCH = datetime.fromtimestamp(0) + timedelta(365 * 3 + 1)

MONTHS = (
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December",
)


def from_GPX(string):
    """Load a time from a string in GPX format."""
    if string is None:
        return None
    return datetime.fromisoformat(string.rstrip("Z"))


def to_string(time: timedelta, exact=False):
    """Convert a time to a nicely formatted string."""
    result = []
    started = False
    if time.days:
        result += [str(time.days), " d "]
        time %= ONE_DAY
        started = True
    if time >= ONE_HOUR or started:
        result += [f"{time // ONE_HOUR:0>2d}", ":"]
        time %= ONE_HOUR
        started = True
    if time >= ONE_MINUTE or started:
        result += [f"{time // ONE_MINUTE:0>2d}", ":"]
        time %= ONE_MINUTE
    secs = time.total_seconds()
    if int(secs) == secs or not exact:
        secs = int(secs)
        result.append(f"{secs:0>2d}")
    else:
        result.append(f"{secs:0>.2f}")
    # Only seconds
    if len(result) == 1:
        result.append(" s")

    return "".join(result).lstrip("0").strip()


def nice(time: datetime):
    """Format a time on two lines neatly."""
    return time.strftime("%A %d %B %Y\n%H:%M")


def round_time(time: datetime) -> datetime:
    """Round a time to the nearest second."""
    new = time - timedelta(microseconds=time.microsecond)
    return new + timedelta(seconds=1) if time.microsecond >= 500000 else new


def to_number(value):
    """Convert a timedelta to seconds, leaving other values untouched."""
    if isinstance(value, timedelta):
        return value.total_seconds()
    return value


def back_name(base, period: str, number=0):
    """Get the name of a year, month or week number back."""
    if period == "year":
        return str(base.year - number)
    elif period == "month":
        return MONTHS[(base.month - number - 1) % 12]
    elif period == "week":
        date = base.date() - number * ONE_WEEK - timedelta(days=base.weekday())
        return f"w/c {date:%d %b}"
    elif period == "day":
        return str((base - ONE_DAY * number).day)
    elif period == "weekday":
        return f"{base - ONE_DAY * number:%A}"
    raise ValueError('period must be "year", "month" or "week"')


def period_difference(base, other, period: str) -> int:
    """
    Determine the number of years/months/weeks between other and base.

    Returns 0 if they are in the same week, 1 if other is in the
    previous week etc.
    """
    if period == "year":
        return base.year - other.year
    if period == "month":
        return base.month - other.month + (base.year - other.year) * 12
    if period == "week":
        value = (base.date() - other.date()).days // 7
        if other.weekday() > base.weekday():
            value += 1
        return value
    if "day" in period:
        return (base.date() - other.date()).days
    raise ValueError('period must be "year", "month" or "week"')


def since_start(base, period: str) -> timedelta:
    """Return the time since the start of the current period."""
    return base - start_of(base, period)


def start_of(base, period: str) -> datetime:
    """Get the start of the current period."""
    if period == "year":
        return datetime(year=base.year, month=1, day=1)
    if period == "month":
        return datetime(year=base.year, month=base.month, day=1)
    if period == "week":
        return (
            datetime(year=base.year, month=base.month, day=base.day)
            - base.weekday() * ONE_DAY
        )
    if "day" in period:
        return datetime(year=base.year, month=base.month, day=base.day)
    raise ValueError('period must be "year", "month", "week" or "day"')


def end_of(base, period: str) -> datetime:
    """Get the end of the current period."""
    if period == "year":
        return start_of(base.replace(year=base.year + 1), period)
    if period == "month":
        if base.month == 12:
            return start_of(base.replace(year=base.year + 1, month=1), period)
        else:
            return start_of(base.replace(month=base.month + 1), period)

    if period == "week":
        return start_of(base + ONE_WEEK, period)
    if "day" in period:
        return start_of(base + ONE_DAY, period)

    raise ValueError('period must be "year", "month" or "week"')


def hours_minutes_seconds(time: timedelta) -> tuple:
    time = time.total_seconds()
    return (time // 3600, *divmod(time % 3600, 60))


def get_periods(minimum, maximum, period: str) -> list:
    """
    Get the periods between minimum and maximum (exclusive).

    Returns a list of (end datetime, name) tuples.
    """
    current = end_of(minimum, period)
    periods = [(current, back_name(current, period, 1))]
    while current <= end_of(maximum, period):
        current = end_of(current, period)
        periods.append((current, back_name(current, period, 1)))
    return periods



"""Classes defining custom dialogs."""

from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt


class FormDialog(QtWidgets.QDialog):
    """A dialog consisiting of a form with OK and Cancel buttons."""

    def __init__(self, form, *args, **kwargs):
        self.form = form
        super().__init__(*args, **kwargs)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.main_layout.addLayout(self.form)

        self.buttons = QtWidgets.QDialogButtonBox(self)
        self.buttons.setStandardButtons(self.buttons.Cancel | self.buttons.Ok)

        self.main_layout.addWidget(self.buttons)

        self.buttons.accepted.connect(self.accept)
        self.buttons.rejected.connect(self.reject)

    def exec(self, initial_values: dict):
        self.form.set_values(initial_values)
        result = super().exec()
        if not result:
            return None
        if result == 1:
            return self.form.values()
        return result


def progress(parent, iterable, text, cancel_text="Cancel"):
    dialog = QtWidgets.QProgressDialog(text, cancel_text, 0, len(iterable), parent)
    dialog.setWindowModality(Qt.WindowModal)
    for done, value in enumerate(iterable):
        dialog.setValue(done)
        if dialog.wasCanceled():
            return
        yield value
    dialog.setValue(len(iterable))



"""
Defines the activity list, which contains all of the user's real data.

This is where computation of summary values should be done.
"""
import dataclasses
import datetime
from collections import Counter
from typing import Optional

from activate import activity, serialise, times, units

LIST_FILENAME = "activity_list.json.gz"
ACTIVITIES_DIR_NAME = "activities"


def from_serial(serial, path):
    return ActivityList((UnloadedActivity(**a) for a in serial), path)


def from_disk(path):
    """Load an activity list from disk, if it exists."""
    try:
        return from_serial(serialise.load(path / LIST_FILENAME), path)
    except FileNotFoundError:
        return ActivityList([], path)


@dataclasses.dataclass
class UnloadedActivity:
    name: str
    sport: str
    flags: dict
    start_time: datetime.datetime
    distance: float
    duration: float
    climb: float
    activity_id: str
    server: Optional[str] = None
    username: Optional[str] = None

    def load(self, path) -> activity.Activity:
        """Get the corresponding loaded Activity from disk."""
        return activity.Activity(**serialise.load(path / f"{self.activity_id}.json.gz"))

    @property
    def list_row(self):
        result = [
            self.name,
            self.sport,
            self.start_time,
            units.DimensionValue(self.distance, "distance"),
        ]
        if self.server is not None:
            result = [self.server, self.username] + result
        return result


class ActivityList(list):
    """A list of activities, which may be loaded."""

    def __init__(self, activities, path=None):
        """Create a list of unloaded activities."""
        self._activities = {}
        self.path = path
        super().__init__(activities)

    def by_id(self, activity_id):
        try:
            return next(a for a in self if a.activity_id == activity_id)
        except StopIteration:
            raise KeyError("No such activity_id")

    def provide_full_activity(self, activity_id, activity_):
        self._activities[activity_id] = activity_

    def get_activity(self, activity_id):
        """Get an activity from its activity_id."""
        if activity_id not in self._activities:
            if self.path is None:
                raise ValueError("Cannot load activity")
            self.provide_full_activity(
                activity_id,
                self.by_id(activity_id).load(self.path / ACTIVITIES_DIR_NAME),
            )
        return self._activities[activity_id]

    def serialised(self):
        return [dataclasses.asdict(a) for a in self]

    def save(self):
        """
        Store the activity list in a file.

        This only stores the list data, not the actual activities.
        """
        serialise.dump(self.serialised(), self.path / LIST_FILENAME, gz=True)

    def add_activity(self, new_activity):
        """
        Add a new activity.

        Also saves the activity to disk.
        """
        self._activities[new_activity.activity_id] = new_activity
        self.append(new_activity.unload(UnloadedActivity))
        new_activity.save(self.path / ACTIVITIES_DIR_NAME)

    def update(self, activity_id):
        """Regenerate an unloaded activity from its loaded version."""
        for i, unloaded_activity in enumerate(self):
            if unloaded_activity.activity_id == activity_id:
                self[i] = self._activities[activity_id].unload(UnloadedActivity)
                break

    def remove(self, activity_id):
        """Remove an activity from all parts of the ActivityList."""
        # Remove from main list
        for a in self:
            if a.activity_id == activity_id:
                super().remove(a)
        # Remove from loaded activities
        if activity_id in self._activities:
            del self._activities[activity_id]

    def filtered(self, activity_types, time_period, now, back=0):
        """
        Get an iterable of the matching activities.

        The activity types must match activity_types and they must have
        taken place in the correct time period. The values for
        time_period are "all time", "year", "month" and "week". A value
        of zero for back gives this year/month/week, 1 gives the
        previous, etc.
        """
        time_period = time_period.casefold()

        return (
            a
            for a in self
            if a.sport in activity_types
            and (
                time_period == "all time"
                or times.period_difference(now, a.start_time, time_period) == back
            )
        )

    def total_distance(self, activities):
        return sum(a.distance for a in activities)

    def total_time(self, activities):
        return sum((a.duration for a in activities), datetime.timedelta())

    def total_activities(self, activities):
        return sum(1 for _ in activities)

    def total_climb(self, activities):
        return sum(a.climb for a in activities if a.climb is not None)

    def eddington(self, activities, unit) -> list:
        """Get a list of days sorted by distance done that day."""
        return sorted(
            sum(
                (
                    Counter(self.get_activity(a.activity_id).track.distance_in_days)
                    for a in activities
                ),
                Counter(),
            ).values(),
            reverse=True,
        )

    def get_progression_data(self, activity_types, time_period, now, key):
        """
        Get the activity dates, along with the total at that point.

        Filter out the wrong activity_types. Evaluate key for each
        activity, and get (dates, totals) in order.
        """
        time_period = time_period.casefold()
        if time_period == "all time":
            data = ([], [])
            total = None
            valid_sorted = sorted(
                self.filtered(activity_types, time_period, now, 0),
                key=lambda x: x.start_time,
            )
            for a in valid_sorted:
                value = key(a)
                if value is None:
                    continue
                if total is None:
                    total = value - value
                data[0].append(a.start_time)
                data[1].append(total)
                total += value
                data[0].append(a.start_time + a.duration)
                data[1].append(total)
            if total is not None:
                data[0].append(now)
                data[1].append(total)

            return (None, [data])

        # Other time periods
        # This is a bit of a hack: all dates are changed to around 1971
        # so that DateTimeAxis can eventually handle them
        periods = []
        result = []
        for back in range(5):
            start = times.start_of(times.EPOCH, time_period)
            data = ([start], [0])
            total = None
            valid_sorted = sorted(
                self.filtered(activity_types, time_period, now, back),
                key=lambda x: x.start_time,
            )
            if not valid_sorted:
                continue
            for a in valid_sorted:
                value = key(a)
                if value is None:
                    continue
                if total is None:
                    total = value - value
                data[0].append(start + times.since_start(a.start_time, time_period))
                data[1].append(total)
                total += value
                data[0].append(
                    start + times.since_start(a.start_time + a.duration, time_period)
                )
                data[1].append(total)
            if back != 0:
                data[0].append(times.end_of(times.EPOCH, time_period))
            else:
                data[0].append(times.EPOCH + times.since_start(now, time_period))
            data[1].append(total)
            result.append(data)
            periods.append(times.back_name(now, time_period, back))

        return (periods[::-1], result[::-1])

    def get_records(
        self, activity_types, time_period, now, distances, progress=lambda x: x
    ):
        records = {}
        activity_ids = {}
        for activity_ in progress(self.filtered(activity_types, time_period, now, 0)):
            for record in self.get_activity(activity_.activity_id).track.get_curve(
                distances
            )[0]:
                if record[0] not in records or records[record[0]][0] > record[1]:
                    records[record[0]] = record[1:] + (activity_.name,)
                    activity_ids[record[0]] = activity_.activity_id
        return (
            [(distance,) + record for distance, record in records.items()],
            activity_ids.values(),
        )

    def get_all_photos(self, activity_types, time_period, now):
        return (
            p
            for a in self.filtered(activity_types, time_period, now)
            for p in self.get_activity(a.activity_id).photos
        )

    def get_all_routes(self, activity_types, time_period, now, progress=lambda x: x):
        result = []
        for activity_ in progress(self.filtered(activity_types, time_period, now)):
            track = self.get_activity(activity_.activity_id).track
            if track.has_position_data:
                result.append(track.lat_lon_list)
        return result

    def get_matching(self, activity_, progress=lambda x: x):
        matching = {activity_.activity_id}
        if not activity_.track.has_position_data:
            return matching
        for other_activity in progress(self):
            if other_activity.activity_id == activity_.activity_id or not (
                other_activity.sport == activity_.sport
                and other_activity.distance / 1.2
                < activity_.distance
                < other_activity.distance * 1.2
            ):
                continue
            if activity_.track.match(
                self.get_activity(other_activity.activity_id).track
            ):
                matching.add(other_activity.activity_id)

        return matching

    def __repr__(self):
        return (
            f"<{self.__class__.__qualname__}"
            f" {super().__repr__()}"
            f" _activities={self._activities!r}>"
        )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'activate/resources/ui/settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_settings(object):
    def setupUi(self, settings):
        settings.setObjectName("settings")
        settings.resize(800, 600)
        self.main_layout = QtWidgets.QVBoxLayout(settings)
        self.main_layout.setObjectName("main_layout")
        self.settings_tabs = QtWidgets.QTabWidget(settings)
        self.settings_tabs.setTabPosition(QtWidgets.QTabWidget.West)
        self.settings_tabs.setObjectName("settings_tabs")
        self.units_tab = QtWidgets.QWidget()
        self.units_tab.setObjectName("units_tab")
        self.units_tab_layout = QtWidgets.QVBoxLayout(self.units_tab)
        self.units_tab_layout.setObjectName("units_tab_layout")
        self.restart_note_label = QtWidgets.QLabel(self.units_tab)
        self.restart_note_label.setObjectName("restart_note_label")
        self.units_tab_layout.addWidget(self.restart_note_label)
        self.unit_system_layout = QtWidgets.QHBoxLayout()
        self.unit_system_layout.setObjectName("unit_system_layout")
        self.system_label = QtWidgets.QLabel(self.units_tab)
        self.system_label.setObjectName("system_label")
        self.unit_system_layout.addWidget(self.system_label)
        self.unit_system = QtWidgets.QComboBox(self.units_tab)
        self.unit_system.setObjectName("unit_system")
        self.unit_system.addItem("")
        self.unit_system.addItem("")
        self.unit_system_layout.addWidget(self.unit_system)
        self.units_tab_layout.addLayout(self.unit_system_layout)
        self.line = QtWidgets.QFrame(self.units_tab)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.units_tab_layout.addWidget(self.line)
        self.custom_units_heading = QtWidgets.QLabel(self.units_tab)
        self.custom_units_heading.setObjectName("custom_units_heading")
        self.units_tab_layout.addWidget(self.custom_units_heading)
        self.settings_tabs.addTab(self.units_tab, "")
        self.servers_tab = QtWidgets.QWidget()
        self.servers_tab.setObjectName("servers_tab")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.servers_tab)
        self.verticalLayout.setObjectName("verticalLayout")
        self.server_table = ServersTable(self.servers_tab)
        self.server_table.setObjectName("server_table")
        self.server_table.setColumnCount(0)
        self.server_table.setRowCount(0)
        self.server_table.horizontalHeader().setDefaultSectionSize(180)
        self.server_table.horizontalHeader().setStretchLastSection(True)
        self.server_table.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.server_table)
        self.add_server_button = QtWidgets.QPushButton(self.servers_tab)
        self.add_server_button.setObjectName("add_server_button")
        self.verticalLayout.addWidget(self.add_server_button)
        self.settings_tabs.addTab(self.servers_tab, "")
        self.main_layout.addWidget(self.settings_tabs)
        self.settings_buttons = QtWidgets.QDialogButtonBox(settings)
        self.settings_buttons.setStandardButtons(
            QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok
        )
        self.settings_buttons.setObjectName("settings_buttons")
        self.main_layout.addWidget(self.settings_buttons)

        self.retranslateUi(settings)
        self.settings_tabs.setCurrentIndex(0)
        self.settings_buttons.accepted.connect(settings.accept)
        self.settings_buttons.rejected.connect(settings.reject)
        self.add_server_button.clicked["bool"].connect(settings.add_server)
        QtCore.QMetaObject.connectSlotsByName(settings)

    def retranslateUi(self, settings):
        _translate = QtCore.QCoreApplication.translate
        settings.setWindowTitle(_translate("settings", "Settings"))
        self.restart_note_label.setText(
            _translate(
                "settings", "A restart may be necessary for changes to take effect."
            )
        )
        self.system_label.setText(_translate("settings", "System"))
        self.unit_system.setItemText(0, _translate("settings", "Metric"))
        self.unit_system.setItemText(1, _translate("settings", "Imperial"))
        self.custom_units_heading.setText(_translate("settings", "Custom Units"))
        self.settings_tabs.setTabText(
            self.settings_tabs.indexOf(self.units_tab), _translate("settings", "Units")
        )
        self.add_server_button.setText(_translate("settings", "Add Server"))
        self.settings_tabs.setTabText(
            self.settings_tabs.indexOf(self.servers_tab),
            _translate("settings", "Servers"),
        )


from activate.app.tables import ServersTable



from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt


def children(item: QtWidgets.QTreeWidgetItem):
    for index in range(item.childCount()):
        yield item.child(index)


class SocialTree(QtWidgets.QTreeWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.header().setVisible(False)

    def set_servers(self, servers, activities):
        self.servers = servers
        self.clear()
        structure = {s.name: set() for s in self.servers}
        for activity in activities:
            for server, username in zip(
                activity.server.split("\n"), activity.username.split("\n")
            ):
                structure[server].add(username)

        for server, users in structure.items():
            item = QtWidgets.QTreeWidgetItem()
            item.setText(0, server)
            item.setCheckState(0, Qt.Checked)
            for user in users:
                sub_item = QtWidgets.QTreeWidgetItem()
                sub_item.setText(0, user)
                sub_item.setCheckState(0, Qt.Checked)
                item.addChild(sub_item)
            self.addTopLevelItem(item)

    def get_enabled_servers(self):
        enabled = []
        for row in range(len(self)):
            item = self.topLevelItem(row)
            if item.checkState(0) == Qt.Checked:
                for sub_item in children(item):
                    if sub_item.checkState(0) == Qt.Checked:
                        enabled.append((item.text(0), sub_item.text(0)))
        return enabled

    def __len__(self):
        return self.topLevelItemCount()
